package core.users;

import core.entities.User;
import core.repositories.UserRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.Optional;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.junit.jupiter.api.Assertions.*;

@DataJpaTest
@ExtendWith(SpringExtension.class)
public class UserRepositoryTests {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private TestEntityManager testEntityManager;

    @Test
    @Sql("classpath:userRepisotoryTestEntries.sql")
    void test_findByUsername() {
        //Situation 1: User with name "FawKes100" is present
        User referenceUser = testEntityManager.find(User.class, 1L);
        Optional<User> findResult = userRepository.findByUsername(referenceUser.getUsername());
        assertTrue(findResult.isPresent());
        assertEquals(referenceUser,findResult.get());

        //Situation 2: No user with the required name is present
        findResult = userRepository.findByUsername("fawkes1001");
        assertFalse(findResult.isPresent());
    }

    @Test
    @Sql("classpath:userRepisotoryTestEntries.sql")
    void test_existsByMail() {
        User referenceUser = testEntityManager.find(User.class, 1L);
        //Situation 1: invalid email format should not matter
        assertFalse(userRepository.existsByEmail("FawKes100"));

        //Situation 2: No user exists with such a mail
        assertFalse(userRepository.existsByEmail("mail@fawkes100.de"));

        //Situation 3: User with this mail exists
        assertTrue(userRepository.existsByEmail(referenceUser.getEmail()));
    }

    @Test
    @Sql("classpath:userRepisotoryTestEntries.sql")
    void test_existsByUsername() {
        User referenceUser = testEntityManager.find(User.class, 1L);
        //Situation 1: No such user
        assertFalse(userRepository.existsByUsername("FawKes100"));

        //Situation 3: User with this name exists
        assertTrue(userRepository.existsByUsername(referenceUser.getUsername()));
    }

    @Test
    @Sql("classpath:userRepisotoryTestEntries.sql")
    void test_deleteUserByName() {
        User referenceUser1 = testEntityManager.find(User.class, 1L);
        User referenceUser2 = testEntityManager.find(User.class, 2L);

        userRepository.deleteByUsername(referenceUser2.getUsername());

        assertThat(testEntityManager.find(User.class, referenceUser2.getId())).isNull();
        assertThat(testEntityManager.find(User.class, referenceUser1.getId())).isNotNull();
    }


    @Test
    @Sql("classpath:userRepisotoryTestEntries.sql")
    void test_countUsers() {
        assertEquals(2, userRepository.countUsers());
    }

}
